/**
 * @file   ppt.h
 * @author Alex Ott <alexott@gmail.com>
 * @date   23 δΕΛ 2004
 * Version: $Id: ppt.h,v 1.1 2006-02-24 17:44:06 vitus Exp $
 * Copyright: Alex Ott
 * 
 * @brief  definitions of .ppt processing functions
 * 
 * 
 */

#ifndef _PPT_H
#define _PPT_H 1

#include <stdio.h>
#include <math.h>

void do_ppt(FILE *input,char *filename);

#endif /* _PPT_H */

